#!/usr/bin/env python3

import os
import configparser
import subprocess
import tempfile
from typing import Dict, List, Optional, Tuple, Any

class ConfigCommand:
    """Base class for Fenrir configuration vmenu commands"""
    
    def __init__(self):
        self.env = None
        self.config = None
        self.settingsFile = None
        
        # Configuration presets from original configure_fenrir.py
        self.presetOptions = {
            'sound.driver': ['genericDriver', 'gstreamerDriver'],
            'speech.driver': ['speechdDriver', 'genericDriver'],
            'screen.driver': ['vcsaDriver', 'dummyDriver', 'ptyDriver', 'debugDriver'],
            'keyboard.driver': ['evdevDriver', 'dummyDriver'],
            'remote.driver': ['unixDriver', 'tcpDriver'],
            'keyboard.charEchoMode': ['0', '1', '2'],
            'general.punctuationLevel': ['none', 'some', 'most', 'all'],
            'general.debugMode': ['File', 'Print'],
            'keyboard.device': ['ALL', 'NOMICE'],
            'screen.encoding': ['auto', 'utf-8', 'cp1252', 'iso-8859-1']
        }
        
        # Help text for various options
        self.helpText = {
            'sound.volume': 'Volume level from 0 (quietest) to 1.0 (loudest)',
            'speech.rate': 'Speech rate from 0 (slowest) to 1.0 (fastest)',
            'speech.pitch': 'Voice pitch from 0 (lowest) to 1.0 (highest)', 
            'speech.capitalPitch': 'Pitch for capital letters from 0 to 1.0',
            'keyboard.charEchoMode': '0 = None, 1 = always, 2 = only while capslock',
            'keyboard.doubleTapTimeout': 'Timeout for double tap in seconds',
            'screen.screenUpdateDelay': 'Delay between screen updates in seconds',
            'general.punctuationLevel': 'none = no punctuation, some = basic, most = detailed, all = everything',
            'general.numberOfClipboards': 'Number of clipboard slots to maintain'
        }
    
    def initialize(self, environment):
        """Initialize with Fenrir environment"""
        self.env = environment
        self.settingsFile = self.env['runtime']['settingsManager'].settingsFile
        self.config = configparser.ConfigParser(interpolation=None)
        self.config.read(self.settingsFile)
    
    def shutdown(self):
        """Cleanup resources"""
        pass
    
    def getDescription(self):
        """Return description for this configuration action"""
        return "Base configuration command"
    
    def run(self):
        """Execute the configuration action - to be overridden"""
        self.presentText("Configuration base class - override run() method")
    
    def presentText(self, text: str, interrupt: bool = True):
        """Present text using Fenrir's speech system"""
        if self.env and 'runtime' in self.env and 'outputManager' in self.env['runtime']:
            self.env['runtime']['outputManager'].presentText(text, interrupt=interrupt)
    
    def presentConfirmation(self, text: str):
        """Present confirmation message that won't be interrupted by menu navigation"""
        self.presentText(text, interrupt=False)
    
    def playSound(self, soundName: str):
        """Play a sound using Fenrir's sound system (deprecated - sounds removed from vmenu)"""
        # Sounds removed from vmenu commands to avoid configuration issues
        pass
    
    def getSetting(self, section: str, option: str, default: str = "") -> str:
        """Get a setting value from configuration"""
        try:
            if section in self.config and option in self.config[section]:
                return self.config[section][option]
            return default
        except Exception:
            return default
    
    def setSetting(self, section: str, option: str, value: str) -> bool:
        """Set a setting value and save configuration"""
        try:
            if section not in self.config:
                self.config[section] = {}
            
            oldValue = self.getSetting(section, option)
            self.config[section][option] = str(value)
            
            # Write configuration file
            with open(self.settingsFile, 'w') as configfile:
                self.config.write(configfile)
            
            # Apply setting immediately if possible
            self.applySettingImmediate(section, option, value, oldValue)
            
            return True
        except Exception as e:
            self.presentText(f"Error saving setting: {str(e)}")
            return False
    
    def applySettingImmediate(self, section: str, option: str, newValue: str, oldValue: str):
        """Apply setting immediately without restart where possible"""
        try:
            # Apply speech settings immediately
            if section == 'speech':
                if option in ['rate', 'pitch', 'volume']:
                    settingsManager = self.env['runtime']['settingsManager']
                    settingsManager.setSetting(section, option, newValue)
                    
            # Apply sound settings immediately  
            elif section == 'sound':
                if option == 'volume':
                    settingsManager = self.env['runtime']['settingsManager']
                    settingsManager.setSetting(section, option, newValue)
                    
        except Exception as e:
            # Silent fail - settings will apply on restart
            pass
    
    def validateInput(self, section: str, option: str, value: str) -> Tuple[bool, str]:
        """Validate user input based on option type and constraints"""
        try:
            # Validate float values (volume, rate, pitch)
            if option in ['volume', 'rate', 'pitch', 'capitalPitch']:
                floatVal = float(value)
                if not 0 <= floatVal <= 1.0:
                    return False, "Value must be between 0 and 1.0"
                return True, str(floatVal)
            
            # Validate integer values
            elif option in ['numberOfClipboards', 'ignoreScreen']:
                intVal = int(value)
                if intVal < 0:
                    return False, "Value must be 0 or greater"
                return True, str(intVal)
            
            # Validate float values with different ranges
            elif option == 'doubleTapTimeout':
                floatVal = float(value)
                if not 0 <= floatVal <= 2.0:
                    return False, "Value must be between 0 and 2.0 seconds"
                return True, str(floatVal)
            
            elif option == 'screenUpdateDelay':
                floatVal = float(value)
                if not 0 <= floatVal <= 1.0:
                    return False, "Value must be between 0 and 1.0 seconds"
                return True, str(floatVal)
            
            # Validate boolean values
            elif self.isBooleanOption(value):
                if value.lower() in ['true', 'false']:
                    return True, value.capitalize()
                return False, "Value must be True or False"
            
            # Validate preset options
            key = f"{section}.{option}"
            if key in self.presetOptions:
                if value in self.presetOptions[key]:
                    return True, value
                return False, f"Value must be one of: {', '.join(self.presetOptions[key])}"
            
            # Default validation - accept any string
            return True, str(value).strip()
            
        except ValueError:
            return False, "Invalid number format"
        except Exception as e:
            return False, f"Validation error: {str(e)}"
    
    def isBooleanOption(self, value: str) -> bool:
        """Check if the current value is likely a boolean option"""
        return value.lower() in ['true', 'false']
    
    def getBooleanSetting(self, section: str, option: str, default: bool = False) -> bool:
        """Get a boolean setting value"""
        value = self.getSetting(section, option, str(default)).lower()
        return value in ['true', '1', 'yes', 'on']
    
    def setBooleanSetting(self, section: str, option: str, value: bool) -> bool:
        """Set a boolean setting value"""
        return self.setSetting(section, option, 'True' if value else 'False')
    
    def toggleBooleanSetting(self, section: str, option: str) -> bool:
        """Toggle a boolean setting and return new value"""
        currentValue = self.getBooleanSetting(section, option)
        newValue = not currentValue
        success = self.setBooleanSetting(section, option, newValue)
        return newValue if success else currentValue
    
    def getFloatSetting(self, section: str, option: str, default: float = 0.0) -> float:
        """Get a float setting value"""
        try:
            return float(self.getSetting(section, option, str(default)))
        except ValueError:
            return default
    
    def setFloatSetting(self, section: str, option: str, value: float) -> bool:
        """Set a float setting value"""
        return self.setSetting(section, option, str(value))
    
    def adjustFloatSetting(self, section: str, option: str, delta: float, 
                          minVal: float = 0.0, maxVal: float = 1.0) -> float:
        """Adjust a float setting by delta amount"""
        currentValue = self.getFloatSetting(section, option)
        newValue = max(minVal, min(maxVal, currentValue + delta))
        success = self.setFloatSetting(section, option, newValue)
        return newValue if success else currentValue
    
    def runCommand(self, cmd: List[str], timeout: int = 10) -> Optional[str]:
        """Run a command and return output"""
        try:
            result = subprocess.run(cmd, capture_output=True, text=True, timeout=timeout)
            return result.stdout.strip() if result.returncode == 0 else None
        except Exception:
            return None
    
    def getSpeechdModules(self) -> List[str]:
        """Get list of available speech-dispatcher modules"""
        output = self.runCommand(['spd-say', '-O'])
        if output:
            lines = output.split('\n')
            return [line.strip() for line in lines[1:] if line.strip()]
        return []
    
    def getModuleVoices(self, module: str) -> List[str]:
        """Get list of voices for a specific speech module"""
        output = self.runCommand(['spd-say', '-o', module, '-L'])
        if output:
            lines = output.split('\n')
            voices = []
            for line in lines[1:]:
                if not line.strip():
                    continue
                if module.lower() == 'espeak-ng':
                    voice = self.processEspeakVoice(line)
                    if voice:
                        voices.append(voice)
                else:
                    voices.append(line.strip())
            return voices
        return []
    
    def processEspeakVoice(self, voiceLine: str) -> Optional[str]:
        """Process espeak-ng voice line format"""
        parts = [p for p in voiceLine.split() if p]
        if len(parts) < 2:
            return None
        langCode = parts[-2].lower()
        variant = parts[-1].lower()
        return f"{langCode}+{variant}" if variant and variant != 'none' else langCode
    
    def testVoice(self, module: str, voice: str, testMessage: str = None) -> bool:
        """Test a voice configuration"""
        if not testMessage:
            testMessage = "This is a voice test. If you can hear this clearly, the voice is working properly."
        
        try:
            # Announce the test
            self.presentText("Testing voice configuration. Listen for the test message.")
            
            # Run voice test
            cmd = ['spd-say', '-o', module, '-y', voice, testMessage]
            result = subprocess.run(cmd, timeout=10)
            
            return result.returncode == 0
        except Exception:
            return False
    
    def backupConfig(self, announce: bool = True) -> tuple[bool, str]:
        """Create a backup of current configuration"""
        try:
            import shutil
            from datetime import datetime
            
            timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
            backupFile = f"{self.settingsFile}.backup_{timestamp}"
            shutil.copy2(self.settingsFile, backupFile)
            
            message = f"Configuration backed up to {backupFile}"
            if announce:
                self.presentText(message, interrupt=False)
            return True, message
        except Exception as e:
            error_msg = f"Error creating backup: {str(e)}"
            if announce:
                self.presentText(error_msg, interrupt=False)
            return False, error_msg
    
    def reloadConfig(self):
        """Reload configuration from file"""
        try:
            self.config.read(self.settingsFile)
            return True
        except Exception:
            return False